/***********************************************************************************
* Intermediate Query Development
* Dave Valentine
* JOINs

Scripts were adapted from scripts written by Jeremy Kadlec at the following URL:
https://www.mssqltips.com/sqlservertip/1667/sql-server-join-example/

***********************************************************************************/

USE AdventureWorks2012;
GO

/*****************************************************************************
INNER JOIN

This query joins Sales Order Details with Products where the Product ID's 
match between both the Sales Order Details with Products tables.
*****************************************************************************/
SELECT  TOP 100 P.ProductID, 
 P.Name, 
 P.ListPrice, 
 P.Size, 
 P.ModifiedDate, 
 SOD.UnitPrice, 
 SOD.UnitPriceDiscount,
 SOD.OrderQty,
 SOD.LineTotal 
FROM Sales.SalesOrderDetail SOD 
INNER JOIN Production.Product P ON SOD.ProductID = P.ProductID 
WHERE SOD.UnitPrice > 1000 
ORDER BY SOD.UnitPrice DESC










/*****************************************************************************
LEFT OUTER JOIN (LEFT JOIN)

This query joins Contact with Sales Person ON the Product ID's and
joins Sales Territory with Sales Person on the Territory ID's.  However, 
results from Sales Territory are returned only if there is a matching 
TerritoryID in the SalesPerson table. 
*****************************************************************************/
SELECT  ContactID = C.BusinessEntityID,
 C.FirstName,
 C.LastName,
 SalesPersonID = SP.BusinessEntityID,
 SP.CommissionPct,
 SP.SalesYTD,
 SP.SalesLastYear,
 SP.Bonus,
 ST.TerritoryID,
 ST.Name,
 ST.[Group],
 ST.SalesYTD
FROM Person.Person C
INNER JOIN Sales.SalesPerson SP ON C.BusinessEntityID = SP.BusinessEntityID
LEFT OUTER JOIN Sales.SalesTerritory ST ON ST.TerritoryID = SP.TerritoryID
ORDER BY ST.TerritoryID, C.LastName
GO

















/*****************************************************************************
RIGHT OUTER JOIN (RIGHT JOIN)

This query will yield the same results as the LEFT OUTER JOIN example.  The
difference here is that the table order is different and we are using a RIGHT
join.  
*****************************************************************************/
SELECT  ContactID = C.BusinessEntityID,
 C.FirstName,
 C.LastName,
 SalesPersonID = SP.BusinessEntityID,
 SP.CommissionPct,
 SP.SalesYTD,
 SP.SalesLastYear,
 SP.Bonus,
 ST.TerritoryID,
 ST.Name, ST.[Group],
 ST.SalesYTD 
FROM Sales.SalesTerritory ST 
RIGHT OUTER JOIN Sales.SalesPerson SP ON ST.TerritoryID = SP.TerritoryID 
INNER JOIN Person.Person C ON C.BusinessEntityID = SP.BusinessEntityID
ORDER BY ST.TerritoryID, C.LastName
GO



















/*****************************************************************************
This query is replacing the original LEFT OUTER JOIN query with a RIGHT OUTER
JOIN.  Notice now that only the records that are available in SalesTerritory
are returned.
*****************************************************************************/
SELECT  ContactID = C.BusinessEntityID,
 C.FirstName,
 C.LastName,
 SalesPersonID = SP.BusinessEntityID,
 SP.CommissionPct,
 SP.SalesYTD,
 SP.SalesLastYear,
 SP.Bonus,
 ST.TerritoryID,
 ST.Name,
 ST.[Group],
 ST.SalesYTD
FROM Person.Person C
INNER JOIN Sales.SalesPerson SP ON C.BusinessEntityID = SP.BusinessEntityID
RIGHT OUTER JOIN Sales.SalesTerritory ST ON ST.TerritoryID = SP.TerritoryID
ORDER BY ST.TerritoryID, C.LastName
GO













/*****************************************************************************
Lets add a new Sales Territory and re-run the RIGHT OUTER JOIN Query.
*****************************************************************************/
INSERT INTO [Sales].[SalesTerritory] ([Name],[CountryRegionCode],[Group],[SalesYTD],[SalesLastYear],[CostYTD],[CostLastYear],[rowguid],[ModifiedDate])
SELECT 'Westeros', 'GB', 'Europe', 5012905.3656,1635823.3967,0.00,0.00, NEWID(), GETDATE()
GO

SELECT  ContactID = C.BusinessEntityID,
 C.FirstName,
 C.LastName,
 SalesPersonID = SP.BusinessEntityID,
 SP.CommissionPct,
 SP.SalesYTD,
 SP.SalesLastYear,
 SP.Bonus,
 ST.TerritoryID,
 ST.Name,
 ST.[Group],
 ST.SalesYTD
FROM Person.Person C
INNER JOIN Sales.SalesPerson SP ON C.BusinessEntityID = SP.BusinessEntityID
RIGHT OUTER JOIN Sales.SalesTerritory ST ON ST.TerritoryID = SP.TerritoryID
ORDER BY ST.TerritoryID, C.LastName
GO


















/*****************************************************************************
FULL OUTER JOIN (FULL JOIN)

This query joins Contact with Sales Person ON the Product ID's and
joins Sales Territory with Sales Person on the Territory ID's.  The
results from Sales Territory are returned regardless if there is a matching 
TerritoryID in the SalesPerson table. 
*****************************************************************************/
SELECT  ContactID = C.BusinessEntityID,
 C.FirstName,
 C.LastName,
 SalesPersonID = SP.BusinessEntityID,
 SP.CommissionPct,
 SP.CommissionPct,
 SP.SalesYTD,
 SP.SalesLastYear,
 SP.Bonus,
 ST.TerritoryID,
 ST.Name,
 ST.[Group],
 ST.SalesYTD
FROM Person.Person C
INNER JOIN Sales.SalesPerson SP ON  C.BusinessEntityID = SP.BusinessEntityID
FULL OUTER JOIN Sales.SalesTerritory ST ON ST.TerritoryID = SP.TerritoryID
ORDER BY ST.TerritoryID, C.LastName
GO








/*****************************************************************************
CROSS JOIN

This query, similar to the INNER JOIN, joins the Sales Order Detail 
with the Produt table.  However, this time it does not return an intersection
between the two tables, this time it is returning a cartesian product.
*****************************************************************************/
SELECT TOP 100 P.ProductID, 
 P.Name, 
 P.ListPrice, 
 P.Size, 
 P.ModifiedDate, 
 SOD.UnitPrice, 
 SOD.UnitPriceDiscount,
 SOD.OrderQty,
 SOD.LineTotal 
FROM Sales.SalesOrderDetail SOD 
CROSS JOIN Production.Product P 
WHERE SOD.UnitPrice > 3500 
ORDER BY SOD.UnitPrice DESC
GO

