/***********************************************************************************
* Intermediate Query Development
* Dave Valentine
* PIVOT Demo
***********************************************************************************/

USE AdventureWorks2012;
GO

/***********************************************************************************
* PART I
* Running the following query returns the the customer sales information.
*
* Requirement: We want to see total sales pivoted by sales person for each year 
* 2005, 2006, 2007, and 2008.  
* Example Output:
	FirstName	LastName	Customer					2005	2006	2007	2008
	Amy			Alberts		Ace Bicycle Supply					74.09		
	Amy			Alberts		Action Bicycle Specialists					80819.73	
	Amy			Alberts		Bulk Discount Store							56147.05	
	Amy			Alberts		Closest Bicycle Store		37883.19	
	Amy			Alberts		Cycles Sales and Repair								15693.37	

***********************************************************************************/
SELECT soh.SalesOrderID
	 , p.FirstName
	 , p.LastName
	 , Customer = Name
	 , TotalDue	 
	 , soh.OrderDate
FROM Sales.SalesOrderHeader soh
JOIN Person.Person p ON p.BusinessEntityID = soh.SalesPersonID
JOIN Sales.Customer c ON c.CustomerID = soh.CustomerID
JOIN Sales.Store s ON s.BusinessEntityID = c.StoreID

/***********************************************************************************
* Running the following query returns the the customer sales information and
* converts the Order date to a year.
***********************************************************************************/
SELECT soh.SalesOrderID
	 , p.FirstName
	 , p.LastName
	 , Customer = Name
	 , TotalDue	 
	 , OrderYear = YEAR(soh.OrderDate)
FROM Sales.SalesOrderHeader soh
JOIN Person.Person p ON p.BusinessEntityID = soh.SalesPersonID
JOIN Sales.Customer c ON c.CustomerID = soh.CustomerID
JOIN Sales.Store s ON s.BusinessEntityID = c.StoreID


/***********************************************************************************
* Running the following query returns the the customer sales information, 
* converts the Order date to a year, and sums the totals.  Now we just need 
* to pivot the results
***********************************************************************************/
SELECT p.FirstName
	 , p.LastName
	 , Customer = s.Name
	 , YearTotal = SUM(soh.TotalDue)
	 , OrderYear = YEAR(soh.OrderDate)
FROM Sales.SalesOrderHeader soh
JOIN Person.Person p ON p.BusinessEntityID = soh.SalesPersonID
JOIN Sales.Customer c ON c.CustomerID = soh.CustomerID
JOIN Sales.Store s ON s.BusinessEntityID = c.StoreID
GROUP BY p.FirstName, p.LastName, s.Name, YEAR(soh.OrderDate)


/***********************************************************************************
* Running the following query returns the the customer sales information totaled 
* and pivoted by the years 2005, 2006, 2007, and 2008
***********************************************************************************/
SELECT PVT.FirstName	
	 , PVT.LastName
	 , PVT.Customer
	 , PVT.[2005]
	 , PVT.[2006]
	 , PVT.[2007]
	 , PVT.[2008]	
FROM (
	SELECT p.FirstName
		 , p.LastName
		 , Customer = s.Name
		 , soh.TotalDue
		 , OrderYear = YEAR(soh.OrderDate)
	FROM Sales.SalesOrderHeader soh
	JOIN Person.Person p ON p.BusinessEntityID = soh.SalesPersonID
	JOIN Sales.Customer c ON c.CustomerID = soh.CustomerID
	JOIN Sales.Store s ON s.BusinessEntityID = c.StoreID
) s
PIVOT (SUM(TotalDue) FOR OrderYear in ([2005],[2006],[2007],[2008])) AS PVT




/***********************************************************************************
* PART II
* Requirement: We want to see total sales pivoted by client for each year 
* 2005, 2006, 2007, and 2008.  
* Example Output:
	Customer					|2005	 |2006	  |2007	|2008
    ----------------------------|--------|--------|-----|------
	A Bike Store				|34533.3 |61390.6 |	    |
	A Great Bicycle Company		|5333.9	 |4291.1  |571.0|	
	A Typical Bike Shop			|36593.2 |57387.8 |	    |
	Acceptable Sales & Service	|472.3	 |944.6	  |     |

Here is a query to get you started
***********************************************************************************/
SELECT Customer = s.Name
		, soh.TotalDue
		, OrderYear = YEAR(soh.OrderDate)
FROM Sales.SalesOrderHeader soh	
JOIN Sales.Customer c ON c.CustomerID = soh.CustomerID
JOIN Sales.Store s ON s.BusinessEntityID = c.StoreID




















/***********************************************************************************
* Your Query should look like this.  Notice by adding columns to the inner most
* query impacts the data that is outputted from the PIVOT. 
***********************************************************************************/
SELECT PVT.Customer
	 , PVT.[2005]
	 , PVT.[2006]
	 , PVT.[2007]
	 , PVT.[2008]	
FROM (
	SELECT Customer = s.Name
		 , soh.TotalDue
		 , OrderYear = YEAR(soh.OrderDate)
	FROM Sales.SalesOrderHeader soh	
	JOIN Sales.Customer c ON c.CustomerID = soh.CustomerID
	JOIN Sales.Store s ON s.BusinessEntityID = c.StoreID
) s
PIVOT (SUM(TotalDue) FOR OrderYear in ([2005],[2006],[2007],[2008])) AS PVT






