/***********************************************************************************
* Intermediate Query Development
* Dave Valentine
* Table Variables Demo SQL Script
***********************************************************************************/
USE AdventureWorks2012;
GO

--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO

/***********************************************************************************
* This demo script demonstrates how to create and read from table variable
* 
***********************************************************************************/
-- Create Table Variable
DECLARE @Products TABLE (
	ProductID INT, 
	ProductName NVARCHAR(100)
)

-- Insert into the Table Variable, same as a persited table
INSERT INTO @Products (ProductID, ProductName) 
VALUES (756,N'Road-450 Red, 44'),(757,N'Road-450 Red, 48');

-- Select data from the Table Variable, same as a persisted table.
SELECT ProductID 
	 , ProductName 
FROM @Products;


/***********************************************************************************
* Now that we have declared the Table Variable above, lets try to read from it 
* again, notice that the table varialble does not exist. 
***********************************************************************************/
-- Select data from the Table Variable
SELECT ProductID 
	 , ProductName 
FROM @Products;
GO


/***********************************************************************************
* This demo script will create our Products table variable again and use it in an 
* INNER JOIN statement, to filter our results.
***********************************************************************************/
-- Create Table Variable
DECLARE @Products TABLE (
	ProductID INT, 
	ProductName NVARCHAR(100)
)

-- Insert into the Table Variable, same as a persited table
INSERT INTO @Products (ProductID, ProductName) 
VALUES (756,N'Road-450 Red, 44'),(757,N'Road-450 Red, 48');

-- Insert the table variable in an INNER JOIN to a query.
SELECT p.productid,
       p.Name,
       MinOrderQty = Min(SOD.orderqty),
       MaxOrderQty = Max(SOD.orderqty),
       SumOrderQty = SUM(SOD.orderqty),
	   NumberOrderQty = COUNT(SOD.orderqty),
       AvgOrderQty = AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
INNER JOIN production.product P ON SOD.productid = P.productid
INNER JOIN @Products tp ON tp.ProductID = p.ProductID
WHERE p.productid in (756,757)
GROUP  BY p.productid,
          p.Name 
GO

/***********************************************************************************
* This demo script will create our Products table variable again and use it in an 
* INNER JOIN statement to filter our results.  Notice that the columns in the 
* Production.Product table are avaialble in the Products Table Variable.
* Do we need the Production.Products table in the Query anymore?
***********************************************************************************/
-- Create Table Variable
DECLARE @Products TABLE (
	ProductID INT, 
	ProductName NVARCHAR(100)
)

-- Insert into the Table Variable, same as a persited table
INSERT INTO @Products (ProductID, ProductName) 
VALUES (756,N'Road-450 Red, 44'),(757,N'Road-450 Red, 48');

-- Insert the table variable in an INNER JOIN to a query.
SELECT tp.productid,
       tp.ProductName,
       MinOrderQty = Min(SOD.orderqty),
       MaxOrderQty = Max(SOD.orderqty),
       SumOrderQty = SUM(SOD.orderqty),
	   NumberOrderQty = COUNT(SOD.orderqty),
       AvgOrderQty = AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
INNER JOIN @Products tp ON tp.ProductID = SOD.ProductID
GROUP  BY tp.productid,
          tp.ProductName;
GO

/***********************************************************************************
* This demo script will create our Products table variable again and use it in an 
* INNER JOIN statement to filter our results.  While manually populating the Table
* Variable is an option, we can also insert data from a SELECT statement, to provide
* additional flexibilty in the products selected.
***********************************************************************************/
-- Create Table Variable
DECLARE @Products TABLE (
	ProductID INT, 
	ProductName NVARCHAR(100)
)

-- Insert into the Table Variable, same as a persited table
INSERT INTO @Products (ProductID, ProductName) 
SELECT ProductID, Name
FROM Production.Product
WHERE Name LIKE '%Road-450%';
--WHERE ProductID BETWEEN 700 AND 800;
--WHERE ReorderPoint < 500;

-- Insert the table variable in an INNER JOIN to a query.
SELECT tp.productid,
       tp.ProductName,
       MinOrderQty = Min(SOD.orderqty),
       MaxOrderQty = Max(SOD.orderqty),
       SumOrderQty = SUM(SOD.orderqty),
	   NumberOrderQty = COUNT(SOD.orderqty),
       AvgOrderQty = AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
INNER JOIN @Products tp ON tp.ProductID = SOD.ProductID
GROUP  BY tp.productid,
          tp.ProductName;
GO

