/***********************************************************************************
* Intermediate Query Development
* Dave Valentine
* Temp Tables Demo SQL Script
***********************************************************************************/
USE AdventureWorks2012;
GO

--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO

/***********************************************************************************
* This demo script demonstrates how to create and read from a temp table
* 
***********************************************************************************/
-- Create Table Variable
CREATE TABLE #Products(
	ProductID INT, 
	ProductName NVARCHAR(100)
)

-- Insert into the Table Variable, same as a persited table
INSERT INTO #Products (ProductID, ProductName) 
VALUES (756,N'Road-450 Red, 44'),(757,N'Road-450 Red, 48');

-- Select data from the Table Variable, same as a persisted table.
SELECT ProductID 
	 , ProductName 
FROM #Products;


/***********************************************************************************
* Now that we have declared the Temp Table above, lets try to read from it 
* again, notice that the temp table exist. 
***********************************************************************************/
-- Select data from the Temp Table
SELECT ProductID 
	 , ProductName 
FROM #Products;
GO


/***********************************************************************************
* This demo script will create our Products temp table again and use it in an 
* INNER JOIN statement to filter our results.  With temp table we can also insert 
* data from a SELECT statement using the INTO statement.
***********************************************************************************/
-- Check to see if the Temp table exists, if it does remove it.
IF OBJECT_ID('tempdb..#Products') IS NOT NULL
	DROP TABLE #Products;

-- Insert into the temp table using an INTO statement
SELECT ProductID
	 , ProductName = name
INTO #Products
FROM Production.Product
WHERE ProductID BETWEEN 700 AND 800;


-- Select using the Temp table.
SELECT tp.productid,
       tp.ProductName,
       MinOrderQty = Min(SOD.orderqty),
       MaxOrderQty = Max(SOD.orderqty),
       SumOrderQty = SUM(SOD.orderqty),
	   NumberOrderQty = COUNT(SOD.orderqty),
       AvgOrderQty = AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
INNER JOIN #Products tp ON tp.ProductID = SOD.ProductID
GROUP  BY tp.productid,
          tp.ProductName;
GO


/***********************************************************************************
* This demo script will create an index on our temp table, check the execution
* plan to ensure the index is being used.
***********************************************************************************/
-- Create the Index
CREATE INDEX IX__ProductsTemp ON #Products(ProductID, ProductNAme)

-- Select using the Temp table.
SELECT tp.productid,
       tp.ProductName,
       MinOrderQty = Min(SOD.orderqty),
       MaxOrderQty = Max(SOD.orderqty),
       SumOrderQty = SUM(SOD.orderqty),
	   NumberOrderQty = COUNT(SOD.orderqty),
       AvgOrderQty = AVG(CAST(SOD.orderqty AS FLOAT))
FROM   sales.salesorderdetail SOD
INNER JOIN #Products tp ON tp.ProductID = SOD.ProductID
GROUP  BY tp.productid,
          tp.ProductName;
GO

/***********************************************************************************
* This demo script will alter the temp table and return the new column.  Notice
* that we are no longer using our index in the execution plan.
***********************************************************************************/
-- Alter the Temp table by adding a inserted date column
ALTER TABLE #Products ADD InsertDate DATETIME

-- Add today's date 
UPDATE #Products SET InsertDate = GETDATE();


-- Select using the Temp table add the new column.
SELECT tp.productid,
       tp.ProductName,
       MinOrderQty = Min(SOD.orderqty),
       MaxOrderQty = Max(SOD.orderqty),
       SumOrderQty = SUM(SOD.orderqty),
	   NumberOrderQty = COUNT(SOD.orderqty),
       AvgOrderQty = AVG(CAST(SOD.orderqty AS FLOAT)),
	   InsertDate
FROM   sales.salesorderdetail SOD
INNER JOIN #Products tp ON tp.ProductID = SOD.ProductID
GROUP  BY tp.productid,
          tp.ProductName
		  , InsertDate;
GO

